<?php

namespace Concrete\Package\JlFormReformDisplay\Block\JlFormReformCompletionList;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Page\Page;

use JtF\FormReform\FormHandlers\HandlerPipelineHelper;
use JtF\FormReform\FormState\InputPersistence;
use JtF\FormReform\Utility\TimeInterval;
use JtF\FormReformDisplay\DisplayBlockController;
use JtF\FormReformDisplay\Reports\ListMakerDefault;

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class Controller extends DisplayBlockController
{
    protected $btTable = 'btJlFormReformCompletionList';
    protected $forms_to_list;
    protected $more_recent_than;
    protected $more_recent_than_u;
    protected $persistence_sources;
    protected $link_to_form_page;
    protected $sort_option;
    protected $progress_option;
    protected $heading;
    protected $incomplete_message;
    protected $completed_message;
    protected $form_page_column_label;
    protected $status_column_label;

    public function getBlockTypeName()
    {
        return t('Completion List');
    }

    public function getBlockTypeDescription()
    {
        return t("Configure a list of forms and pages, showing the user's progress at completing the forms.");
    }

    public function on_start()
    {
        parent::on_start();

        if (!empty($this->persistence_sources)) {
            if (!is_array($this->persistence_sources)) {
                $this->persistence_sources = preg_split("/[^0-9a-z]+/i", $this->persistence_sources);
            }
            $this->set('persistence_sources', $this->persistence_sources);
        }

        if (!empty($this->forms_to_list) && is_string($this->forms_to_list)) {
            $this->forms_to_list = json_decode($this->forms_to_list, true);
            $this->set('forms_to_list', $this->forms_to_list);
        }
    }

    public function add()
    {
        $this->set('incomplete_message', t('ToDo'));
        $this->set('completed_message', t('Done'));
        $this->set('form_page_column_label', t('Form'));  // this is actually a page, but saying 'Form' makes more sense to visitors
        $this->set('status_column_label', t('Status'));
    }


    public function validate($args)
    {
        $error = $this->app->make('helper/validation/error');
        if (empty($args['forms_to_list']) || !is_array($args['forms_to_list'])) {
            $error->add(t('Please specify at least one page with a form on it.'));
        } else {
            foreach ($args['forms_to_list'] as $form_to_list) {
                if (empty($form_to_list['cid'])) {
                    $error->add(t('Each page needs to be selected'));
                    break;
                }
                $target_page = Page::getByID($form_to_list['cid'] ?? null);
                if (!is_object($target_page)) {
                    $error->add(t('Page not found'));
                    break;
                }
            }
        }
        return $error;
    }

    public function save($args)
    {
        if (!empty($args['forms_to_list']) && is_array($args['forms_to_list'])) {
            foreach ($args['forms_to_list'] as &$form_to_list) { // IMPORTANT - need to coerce to ensure handle before json
                $hph = $this->app->make(HandlerPipelineHelper::class);
                $form_to_list['form_name'] = $hph->handlify_anything(!empty($form_to_list['form_name']) ? $form_to_list['form_name'] : 'form_reform');
            }
            $args['forms_to_list'] = json_encode($args['forms_to_list'], JSON_UNESCAPED_SLASHES + JSON_UNESCAPED_UNICODE);
        } else {
            $args['forms_to_list'] = $args['forms_to_list'] ?? '';
        }

        if (!empty($args['persistence_sources'])) {
            $args['persistence_sources'] = implode(',', $args['persistence_sources']);
        } else {
            $args['persistence_sources'] = '';
        }


        parent::save($args);
    }

    public function getLinkOptions()
    {
        return [
            ''             => t('No link'),
            'missing'      => t('Link only to forms that have not been completed'),
            'done'         => t('Link only to forms that have been completed'),
            'done_missing' => t('Link to all forms'),
        ];
    }

    public function getSortOptions()
    {
        return [
            ''        => t('As specified in Form tab (default)'),
            'missing' => t('Missing before submitted'),
            'done'    => t('Submitted before missing'),
        ];
    }

    public function getProgressOptions()
    {
        return [
            ''             => t('No progress display'),
            'bar'          => t('Percentage progress bar'),
            'bar_fraction' => t('Fraction progress bar'),
        ];
    }

    /*
     * Called from View.php to get results list.
     */
    public function getResults()
    {
        if (empty($this->forms_to_list)) {
            return;
        }
        if (is_string($this->forms_to_list)) {
            $this->forms_to_list = json_decode($this->forms_to_list, true);
        }
        if (!empty($this->persistence_sources) && !is_array($this->persistence_sources)) {
            $this->persistence_sources = preg_split("/[^0-9a-z]+/i", $this->persistence_sources);
        }

        if (!empty($this->more_recent_than) && !empty($this->more_recent_than_u)) {
            $ti = new TimeInterval();
            $cutoff_t_sql = $ti->calculateOffsetSQLTime($this->more_recent_than, $this->more_recent_than_u);
        } else {
            $cutoff_t_sql = null;
        }

        /*
         * In the edit, a list of pages with the form on the page is specified.
         * We now loop through that and find the latest form results for the
         * current user and current save handlers.
         */
        $latest_forms = [];
        foreach ($this->forms_to_list as $form_to_list) {
            if (empty($form_to_list['cid'])) {
                continue;
            }
            $target_page = Page::getByID($form_to_list['cid'] ?? null);
            if (!is_object($target_page)) {
                continue;
            }

            $input_persistence = new InputPersistence('from_form_page', $this->persistence_sources ?? null,
                                                      !empty($form_to_list['form_name']) ? $form_to_list['form_name'] : 'form_reform');


            $submitted = $input_persistence->getLatestSubmission($form_to_list['cid'], $cutoff_t_sql);

            $latest_forms[] = [
                'cid'       => $form_to_list['cid'],
                'page'      => $target_page,
                'form_name' => $form_to_list['form_name'] ?? 'form_reform',
                'submitted' => $submitted ?? false,
            ];
        }

        /*
         * Default sort is the sequence of the pages/forms in the edit dialogue.
         * That can be biased to put missing opr done at the top. Other sort remains
         * as per edit.
         */
        if (!empty($this->sort_option)) {
            $missing = [];
            $done = [];
            foreach ($latest_forms as $form_to_list) {
                if (empty($form_to_list['submitted'])) {
                    $missing[] = $form_to_list;
                } else {
                    $done[] = $form_to_list;
                }
            }
            if ($this->sort_option === 'missing') {
                $latest_forms = array_merge($missing, $done);
            } else {
                $latest_forms = array_merge($done, $missing);
            }
        }
        return $latest_forms;
    }
}
