<?php

namespace Concrete\Package\JlFormReformDisplay;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Block\BlockType\BlockType;
use Concrete\Core\Block\BlockType\Set;
use Concrete\Core\Package\Package;
use Concrete\Core\Package\PackageService;
use Concrete\Core\Support\Facade\Config;
use Doctrine\ORM\EntityManagerInterface;

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class Controller extends Package
{

    protected $pkgHandle          = 'jl_form_reform_display';
    protected $appVersionRequired = '8.5.12';
    protected $pkgVersion         = '9.1.2';

    /*
     * v9.0.1 - Change to UCP detection for more flexible install.
     * v9.0.2 - Display user attribute textarea. Improve display for list values.
     * v9.0.3 - Completion list block. Fix linking to page from results blocks.
     * v9.0.4 - Fix sequencing issue with updating Universal Content Puller integration
     * v9.0.5 - Support display of forms saved to attributes, files and csv. Fix 'Cid' to 'cid'.
     * v9.0.6 - Fix object display for php8.2 in default results list
     * v9.0.7 - Update minimum dependency on Form Reform. Extended install information.
     * v9.1.0 - Refactored for ease of further extension. Extended format options.
     *          Display templates for plain content. Support for display extensions
     *          addons such as Reviews. Query builder to filter default store listings.
     *          Support display of signatures. Optional integration of Last Updated for
     *          date formatting. Provision of result ID in listings for use with Result Picker.
     *          Display session ID for super admin.
     * v9.1.1 - Output formats for Fid - image, download, etc. Cid - page link. Fix php7.4
     *          back compatibility.
     * v9.1.2 - Add missing 'continue' to custom display option handling.
     *
     */

    public function getPackageDependencies()
    {
        return [
            'jl_form_reform' => '9.3.15',
        ];
    }
    public function getPackageAutoloaderRegistries()
    {
        $auto = [
            'src/JtF/FormReformDisplay' => '\\JtF\\FormReformDisplay',
        ];
        /*
         * Only if UCP is installed
         */
        $pkg = $this->app->make(PackageService::class)->getByHandle('jl_universal_content_puller');
        if (is_object($pkg)) {
            $auto['src/JtF/UCP/ContentSources'] = '\\JtF\\UCP\\ContentSources';
        }
        return $auto;
    }

    public function getPackageName()
    {
        return t('Form Reform Display');
    }

    public function getPackageDescription()
    {
        return t('List and display form submissions from Form Reform.');
    }

    public function install()
    {
        $pkg = parent::install();
        $this->do_install($pkg);
    }

    public function upgrade()
    {
        parent::upgrade();
        $this->do_install($this);
    }

    private function do_install($pkg)
    {
        $installer = $this->app->make('JtFResources\Install\Installer');
        $installer->addBlockTypeSet('form_reform_display', t('Form Reform Display'));
        $installer->installBlocks($pkg, $this->list_blocks());
        if (class_exists('JtF\UCP\ContentSources\ContentSourcePluginsController')) {
            $this->app->make('JtF\UCP\ContentSources\ContentSourcePluginsController')->listAll(true);
        }
        /*
         * Dashboard pages - currently WIP, so disabled from install
         */
        //$installer->installDashboardPages($pkg, $this->list_dashboard_pages());
    }

    public function uninstall()
    {
        $db = $this->app->make('database')->connection();
        $entityManager = $this->app->make(EntityManagerInterface::class);

        /*
         * By explicitly uninstalling all the block types first, we can come back and try again if it fails
         * such as running out of time.
         */
        foreach ($this->list_blocks() as $bt_handle => $bt_table) {
            $bt = BlockType::getByHandle($bt_handle);
            if (is_object($bt) && $bt->canUnInstall()) {
                $bt->delete();
                $db->executeQuery('DROP TABLE IF EXISTS ' . $bt_table);
            }
        }
        $entityManager->flush();

        parent::uninstall();

        $entityManager->flush();

        foreach (['form_reform_display'] as $set_handle) {
            $block_type_set = Set::getByHandle($set_handle);
            if (!empty($block_type_set) && is_object($block_type_set)) {
                $num_bts_in_set = $this->fetchOne('SELECT COUNT(*) FROM BlockTypeSetBlockTypes WHERE btsID = ? ',
                                                  [$block_type_set->getBlockTypeSetID()]);
                if (empty($num_bts_in_set)) {
                    $block_type_set->delete();
                }
            }
        }
        if (class_exists('JtF\UCP\ContentSources\ContentSourcePluginsController')) {
            $this->app->make('JtF\UCP\ContentSources\ContentSourcePluginsController')->listAll(true);
        }
    }

    protected function fetchOne($q, $p = [])
    {
        $db = $this->app->make('database')->connection();
        if (version_compare(Config::get('concrete.version'), 9.0, '<')) {
            return $db->fetchColumn($q, $p);
        } else {
            return $db->fetchOne($q, $p);
        }
    }

    private function list_blocks()
    {
        return [
            'jl_form_reform_list_results'          => 'btJlFormReformListResults',
            'jl_form_reform_display_result'        => 'btJlFormReformDisplayResult',
            'jl_form_reform_list_cookie_results'   => 'btJlFormReformListCookieResults',
            'jl_form_reform_display_cookie_result' => 'btJlFormReformDisplayCookieResult',
            'jl_form_reform_completion_list'       => 'btJlFormReformCompletionList',
        ];
    }

    private function list_dashboard_pages()
    {
        return [
            '/dashboard/reports/form_reform/form_reform_report'
            => [
                'cName'        => t('Form Report Generator'),
                'cDescription' => t('Report generator for default Form Reform submissions.'),
            ],

        ];
    }
}
