<?php

namespace JtF\FormReformDisplay\Reports;

defined('C5_EXECUTE') or die('Access Denied.');


use Concrete\Core\User\User;
use JtF\FormReformDisplay\Reports\DateUtils;

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class ArrayStoreLister
{

    /*
     * Given a list of results, as provided by cookie or session store, filters and extracts from that list.
     * TODO
     *  May be extenended in the future to provide pagination. However, the core lister/paginator interface
     *  is not suited to doing such and cookie/session stores should not be long, so no reason to go to
     *  the added complexity at present.
     */

    protected $list_of_results;
    protected $form_name;
    protected $filter_configuration;

    public function __construct($list_of_results, $filter_configuration, $form_name = null)
    {
        $this->list_of_results = $list_of_results;
        $this->form_name = $form_name;
        $this->filter_configuration = $filter_configuration;
    }

    public function get_listing($order_by, $offset = 0, $length = 1)
    {
        if (empty($this->list_of_results)) {
            return;
        }

        $results = [];
        foreach ($this->list_of_results as $ix => $result) {

            /*
             * Form name filter
             */
            if (!empty($this->form_name) &&
                (string)$this->form_name !== (string)($result['form_name'] ?? 'form_reform')
            ) {
                continue;
            }

            if (empty($result['id']) && is_array($result)) {
                $result['id'] = $ix + 1;
            }

            if (empty($this->filter_configuration)) {
                $results[] = $result;

            } else {

                /*
                 * Page filters. Make sure we have integers to compare (or we could get an issue
                 * of comparing an integer to a string, both the same number, and fail!)
                 */
                if (!empty($this->filter_configuration['cid']) &&
                    (int)$this->filter_configuration['cid'] !== (int)($result['cid'] ?? -1)
                ) {
                    continue;
                }

                /*
                 * Do we need to filter by user - the current user or by ID
                 */
                if (!empty($this->filter_configuration['uid'])) {
                    $session_id = app()->make('session')->getID();
                    if ($this->filter_configuration['uid'] === 'current') {
                        $u = new User();
                    } else {
                        $u = User::getByUserID($this->filter_configuration['uid']);
                    }
                    $uid = (int)$u->getUserID();

                    if (is_object($u) && $u->isRegistered() &&
                        !empty($result['uid']) && !empty($uid) &&
                        $result['uid'] == $uid
                    ) {
                        //OK - we want it so far
                    } elseif (!empty($session_id) && !empty($result['session_id']) &&
                        $result['session_id'] === $session_id
                    ) {
                        //OK - we still want it
                    } else {
                        continue; // not the current user or session
                    }
                }

                /*
                 * Date filter
                 */
                switch ($this->filter_configuration['date_filter_mode'] ?? null) {
                    case 'today':
                        $on_date = date('Y-m-d');
                        if (!empty($result['DateCreated']) &&
                            !empty($on_date) &&
                            strpos($result['DateCreated'], $on_date) === 0
                        ) {
                            $results[] = $result;
                        }
                        break;

                    case 'past':
                        $after_date = date('Y-m-d', strtotime(sprintf('-%d days', $this->filter_configuration['number_days'] ?? null)));
                        $before_date = date('Y-m-d');
                        [$after_date, $before_date,] = DateUtils::date_sort([$after_date, $before_date]);
                        $before_date = DateUtils::end_of_day($before_date);
                        $after_date = DateUtils::start_of_day($after_date);
                        if (!empty($result['DateCreated']) &&
                            !empty($before_date) &&
                            !empty($after_date)
                        ) {
                            if ($result['DateCreated'] <= $before_date &&
                                $result['DateCreated'] >= $after_date
                            ) {
                                $results[] = $result;
                            }
                        }
                        break;

                    case 'on':
                        $on_date = $this->filter_configuration['date1'] ?? null;
                        if (!empty($result['DateCreated']) &&
                            !empty($on_date) &&
                            strpos($result['DateCreated'], $on_date) === 0
                        ) {
                            $results[] = $result;
                        }
                        break;

                    case 'between':
                        $before_date = $this->filter_configuration['date2'] ?? null;
                        $after_date = $this->filter_configuration['date1'] ?? null;
                        [$after_date, $before_date,] = DateUtils::date_sort([$after_date, $before_date]);
                        $before_date = DateUtils::end_of_day($before_date);
                        $after_date = DateUtils::start_of_day($after_date);
                        if (!empty($result['DateCreated']) &&
                            !empty($before_date) &&
                            !empty($after_date)
                        ) {
                            if ($result['DateCreated'] <= $before_date &&
                                $result['DateCreated'] >= $after_date
                            ) {
                                $results[] = $result;
                            }
                        }
                        break;

                    case 'all':
                    default:
                        $results[] = $result;
                        break;
                }
            }

        }
        if (empty($results)) {
            return;
        }
        switch ($order_by ?? null) {
            case 'asc_id':
                break;
            case 'desc_id':
            default:
                $results = array_reverse($results);
                break;
        }

        if (!empty($length) && count($results) > $length) {
            $results = array_slice($results, $offset, $length);
        }

        return $results;
    }
}