<?php

namespace JtF\FormReformDisplay\Reports;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Support\Facade\Config;
use Concrete\Core\User\User;
use Concrete\Core\Page\Page;
use DateTime;
use Concrete\Core\Application\ApplicationAwareInterface;
use Concrete\Core\Application\ApplicationAwareTrait;
use Concrete\Core\Support\Facade\Application;

use JtF\FormReform\Reports\Entity\FormReformDefault as FormReformDefaultEntity;
use JtF\FormReform\Reports\FormReformDefaultList;
use JtF\FormReformDisplay\Reports\DateUtils;

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class ListMakerDefault implements ApplicationAwareInterface
{
    use ApplicationAwareTrait;

    protected $form_name;
    protected $filter_configuration;
    protected $number_results;
    protected $order_by;
    protected $paginate;
    protected $pagination_str;
    protected $offset_page;
    protected $list;

    public function __construct($form_name, $filter_configuration, $number_results = 0, $order_by = 'desc_id', $paginate = false)
    {
        $this->filter_configuration = $filter_configuration;
        $this->form_name = $form_name;
        $this->number_results = $number_results;
        $this->order_by = $order_by;
        $this->paginate = $paginate;
    }

    public function set_pagination_page($offset_page = 0)
    {
        if ($offset_page > 1) {
            $this->offset_page = $offset_page;
            $this->paginate = true;
        }
    }

    public function set_pagination($pagination_str)
    {
        $this->pagination_str = $pagination_str;
    }

    public function get_pagination()
    {
        return $this->pagination_str;
    }

    public function has_pagination()
    {
        return !empty($this->paginate) && !empty($this->pagination_str);
    }

    public function get_paginated_listing()
    {
        $this->prepare_list();

        if (!empty($this->number_results)) {
            $this->list->setItemsPerPage($this->number_results);
            $paginator = $this->list->getPagination();
            if (!empty($this->offset_page)) {
                $paginator->setCurrentPage($this->offset_page);
            }
            $results = $paginator->getCurrentPageResults();

            if (!empty($this->paginate) && $paginator->haveToPaginate()) {
                $pagination = $paginator->renderDefaultView();
                $this->set_pagination($pagination);
            }
        } else {
            $results = $this->list->getResults();
        }

        return $results;
    }


    public function prepare_list()
    {
        $this->make_list();
        $this->set_sorting();
        $this->set_date_filter();
        $this->set_user_filter();
        $this->set_page_filter();
        $this->set_json_filter();

        /*
         * In case of diagnostic
         */
        /*
        echo "<pre>";
        echo "\n\n\n\n\n\n\n\n**********".__CLASS__."::". __FUNCTION__."[". __LINE__. "]**********\n";
        echo h(print_r($this->list->getQueryObject()->getSQL(),true));
        echo "\n====================\n";
        echo h(print_r($this->list->getQueryObject()->getParameters(),true));
        echo "\n====================";
        echo "</pre>";
        */
        return $this->list;

    }

    public function make_list()
    {
        $this->list = $this->app->make(FormReformDefaultList::class);
        $this->list->filterByFormName($this->form_name);
        $this->list->ignorePermissions();
        return $this->list;
    }

    public function set_sorting()
    {
        switch ($this->order_by) {
            case 'asc_id':
                $this->list->sortBy('id', 'asc');
                break;
            case 'desc_id':
            default:
                $this->list->sortBy('id', 'desc');
                break;
        }
        return $this->list;
    }

    public function set_date_filter()
    {
        switch ($this->filter_configuration['date_filter_mode']) {
            case 'today':
                $on_date = date('Y-m-d');
                if (!empty($on_date)) {
                    $this->list->filterByDateCreated($on_date);
                }
                break;

            case 'past':
                $after_date = date('Y-m-d', strtotime(sprintf('-%d days', $this->filter_configuration['number_days'])));
                $before_date = date('Y-m-d');
                [$after_date, $before_date,] = DateUtils::date_sort([$after_date, $before_date]);
                $before_date = DateUtils::end_of_day($before_date);
                $after_date = DateUtils::start_of_day($after_date);
                if (!empty($before_date) && !empty($after_date)) {
                    $this->list->filterByDateCreatedBefore($before_date);
                    $this->list->filterByDateCreatedAfter($after_date);
                }
                break;

            case 'on':
                $on_date = $this->filter_configuration['date1'];
                if (!empty($on_date)) {
                    $this->list->filterByDateCreated($on_date);
                }
                break;

            case 'between':
                $before_date = $this->filter_configuration['date2'];
                $after_date = $this->filter_configuration['date1'];
                [$after_date, $before_date,] = DateUtils::date_sort([$after_date, $before_date]);
                $before_date = DateUtils::end_of_day($before_date);
                $after_date = DateUtils::start_of_day($after_date);
                if (!empty($before_date) && !empty($after_date)) {
                    $this->list->filterByDateCreatedBefore($before_date);
                    $this->list->filterByDateCreatedAfter($after_date);
                }
                break;

            case 'all':
            default:
                break;
        }
        return $this->list;
    }

    public function set_user_filter()
    {
        switch ($this->filter_configuration['user_filter_mode']) {
            case 'current':
                $u = new User();
                $uid = $u->getUserID();
                if ($uid > 0) {
                    $this->list->filterByUid($uid);
                } else {
                    $session_id = $this->app->make('session')->getID();
                    $this->list->filterBySessionID($session_id);
                }
                break;

            case 'selected':
                $u = User::getByUserID($this->filter_configuration['uid']);
                if (is_object($u)) {
                    $this->list->filterByUid($this->filter_configuration['uid']);
                } else {
                    return; // no such current user, so cannot match.
                }
                break;

            case 'not_registered':
                $this->list->filterByUid(0);
                break;

            case 'registered':
                $this->list->filterByNotUid(0);
                break;

            case 'none':
            default:
                break;
        }
        return $this->list;
    }

    public function set_page_filter()
    {
        switch ($this->filter_configuration['page_filter_mode']) {
            case 'current':
                $page = Page::getCurrentPage();
                $cid = $page->getCollectionID();
                $this->list->filterByCid($cid);
                break;

            case 'selected':
                $page = Page::getByID($this->filter_configuration['cid']);
                if (is_object($page) && !$page->isError()) {
                    $this->list->filterByCid($this->filter_configuration['cid']);
                } else {
                    return;
                }
                break;

            /*
             * TODO - this could be tricky. Will need to join with the page path, so a
             *  considerably more complex query
             */
            case 'beneath_current':
            case 'beneath_selected':
                break;


            case 'none':
            default:
                break;
        }
        return $this->list;
    }


    public function set_json_filter()
    {
        if(empty($this->filter_configuration['json_filter'])){
            return;
        }
        if(is_scalar($this->filter_configuration['json_filter'])){
            $this->filter_configuration['json_filter'] = json_decode($this->filter_configuration['json_filter'], true);
        }
        if(empty($this->filter_configuration['json_filter'])){
            return;
        }

        foreach ($this->filter_configuration['json_filter'] as $json_filter){
            if(empty($json_filter['saved_form_field'])){
                continue;
            }
            $this->list->filterByFormDataJSON(
                $json_filter['value_to_compare'],
                $json_filter['saved_form_field'],
                !empty($json_filter['case_invariant']),
                $json_filter['json_comparison']
            );
        }

        return $this->list;
    }


    /*
     * A list of all form names, default name 'form_reform'
     * at the top of the list.
     */
    public static function form_names()
    {
        $db = Application::getFacadeApplication()->make('database')->connection();
        $form_names = $db->fetchAll("SELECT DISTINCT `form_name` AS 'FormName' FROM FormReformDefault ORDER BY `form_name` ASC");
        if (empty($form_names)) {
            return [Config::get('jl_form_reform.default_form_name')];
        }
        $form_names = array_map(function ($a) {
            return $a['FormName'];
        }, $form_names);
        if (($k = array_search(Config::get('jl_form_reform.default_form_name'), $form_names)) !== false) {
            unset($form_names[$k]);
            array_unshift($form_names, Config::get('jl_form_reform.default_form_name'));
        }
        return $form_names;
    }


    public static function metadata_columns($data_store_is_user_specific = false)
    {
        $metadata_columns = [
            'id'            => t('Submission ID'),
            'form_name'     => t('Form Name'),
            'date_created'  => t('Date Created'),
            'date_modified' => t('Date Modified'),
            'cid'           => t('From Page'),
            'uid'           => t('By User'),
            'uid_modified'  => t('Modified by User'),
            'ip'            => t('IP Address')
        ];
        $u= new User();
        if($u->isSuperUser()){
            $metadata_columns['session'] = t('%sSession ID%s', '<span class="text-danger">', '</span');
        }

        if (!empty($data_store_is_user_specific)) {
            unset($metadata_columns['uid']);
            unset($metadata_columns['uid_modified']);
            unset($metadata_columns['ip']);
        }
        return $metadata_columns;
    }

    public static function filter_date_default()
    {
        return 'all';
    }

    public static function filter_date_options()
    {
        $filter_date_options = [
            'all'     => t('Any'),
            'today'   => t('Today'),
            'past'    => t('Days before today'),
            'on'      => t('Specific date'),
            'between' => t('Between'),
            /*
             * TODO - modified variants of the above
             */
        ];
        return $filter_date_options;
    }

    public static function filter_user_default()
    {
        return 'none';
    }

    public static function filter_user_options()
    {
        $filter_user_options = [
            'none'           => t('No filter'),
            'not_registered' => t('Guests'),
            'registered'     => t('Any registered user'),
            'current'        => t('Current user'),
            'selected'       => t('Selected user'),
        ];
        return $filter_user_options;
    }

    public static function filter_page_default()
    {
        return 'none';
    }

    public static function filter_page_options($exclude_options = null)
    {
        $filter_page_options = [
            'none'     => t('No filter'),
            'current'  => t('Current page'),
            //'beneath_current'  => t('Any page beneath the current'), // TODO
            'selected' => t('Selected page'),
            //'beneath_selected' => t('Any page beneath the selected'), // TODO
        ];
        if (!empty($exclude_options)) {
            if (!is_string($exclude_options)) {
                $exclude_options = implode(' ', $exclude_options);
            }
        }
        $filter_page_options = array_filter($filter_page_options, function ($k) use ($exclude_options) {
            return (strpos($exclude_options, $k) === false);
        },                                  ARRAY_FILTER_USE_KEY);

        return $filter_page_options;
    }


}